# On Python 2, a deadlock is possible if we import a module that runs gevent's getaddrinfo
# with a unicode hostname, which starts Python's getaddrinfo on a thread, which
# attempts to import encodings.idna but blocks on the import lock. Verify
# that gevent avoids this deadlock.

import getaddrinfo_module # pylint:disable=import-error
del getaddrinfo_module  # fix pyflakes



from __future__ import print_function, absolute_import
from gevent import monkey; monkey.patch_all()
import signal

import socket
from time import sleep

import gevent
from gevent.server import StreamServer

import gevent.testing as greentest
from gevent.testing import util

@greentest.skipOnLibuvOnCIOnPyPy("Timing issues sometimes lead to connection refused")
class Test(util.TestServer):
    example = 'portforwarder.py'
    # [listen on, forward to]
    example_args = ['127.0.0.1:10011', '127.0.0.1:10012']

    if greentest.WIN:
        from subprocess import CREATE_NEW_PROCESS_GROUP
        # Must be in a new process group to use CTRL_C_EVENT, otherwise
        # we get killed too
        start_kwargs = {'creationflags': CREATE_NEW_PROCESS_GROUP}

    def after(self):
        if greentest.WIN:
            self.assertIsNotNone(self.popen.poll())
        else:
            self.assertEqual(self.popen.poll(), 0)

    def _run_all_tests(self):
        log = []

        def handle(sock, _address):
            while True:
                data = sock.recv(1024)
                print('got %r' % data)
                if not data:
                    break
                log.append(data)

        server = StreamServer(self.example_args[1], handle)
        server.start()
        try:
            conn = socket.create_connection(('127.0.0.1', 10011))
            conn.sendall(b'msg1')
            sleep(0.1)
            # On Windows, SIGTERM actually abruptly terminates the process;
            # it can't be caught. However, CTRL_C_EVENT results in a KeyboardInterrupt
            # being raised, so we can shut down properly.
            self.popen.send_signal(getattr(signal, 'CTRL_C_EVENT', signal.SIGTERM))
            sleep(0.1)

            conn.sendall(b'msg2')
            conn.close()

            with gevent.Timeout(2.1):
                self.popen.wait()
        finally:
            server.close()

        self.assertEqual([b'msg1', b'msg2'], log)


if __name__ == '__main__':
    greentest.main()



# -*- coding: utf-8 -*-
# copyright 2018 gevent contributors. See LICENSE for details.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import gc


import gevent.testing as greentest
from gevent._ident import IdentRegistry
from gevent._compat import PYPY

class Target(object):
    pass

class TestIdent(greentest.TestCase):

    def setUp(self):
        self.reg = IdentRegistry()

    def tearDown(self):
        self.reg = None

    def test_basic(self):
        target = Target()
        self.assertEqual(0, self.reg.get_ident(target))
        self.assertEqual(1, len(self.reg))

        self.assertEqual(0, self.reg.get_ident(target))
        self.assertEqual(1, len(self.reg))

        target2 = Target()
        self.assertEqual(1, self.reg.get_ident(target2))
        self.assertEqual(2, len(self.reg))

        self.assertEqual(1, self.reg.get_ident(target2))
        self.assertEqual(2, len(self.reg))

        self.assertEqual(0, self.reg.get_ident(target))

        # When an object dies, we can re-use
        # its id. Under PyPy we need to collect garbage first.
        del target
        if PYPY:
            for _ in range(3):
                gc.collect()

        self.assertEqual(1, len(self.reg))

        target3 = Target()
        self.assertEqual(1, self.reg.get_ident(target2))
        self.assertEqual(0, self.reg.get_ident(target3))
        self.assertEqual(2, len(self.reg))

    @greentest.skipOnPyPy("This would need to GC very frequently")
    def test_circle(self):
        keep_count = 3
        keepalive = [None] * keep_count

        for i in range(1000):
            target = Target()
            # Drop an old one.
            keepalive[i % keep_count] = target
            self.assertLessEqual(self.reg.get_ident(target), keep_count)


@greentest.skipOnPurePython("Needs C extension")
class TestCExt(greentest.TestCase):

    def test_c_extension(self):
        self.assertEqual(IdentRegistry.__module__,
                         'gevent._gevent_c_ident')




if __name__ == '__main__':
    greentest.main()
